@@ -86,7 +86,7 @@ class SignatureConfiguration:
 @dataclass(frozen=True)
 class VerifyResult:
     """
-    This is a dataclass representing structured data returned by ``signxml.XMLVerifier.verify()``. The results of a
+    This is a dataclass representing structured data returned by :func:`signxml.XMLVerifier.verify`. The results of a
     verification contain the signed bytes, the parsed signed XML, and the parsed signature XML. Example usage:
 
         verified_data = signxml.XMLVerifier().verify(input_data).signed_xml
@@ -239,7 +239,7 @@ class XMLVerifier(XMLSignatureProcessor):
         Verify the XML signature supplied in the data and return a list of :class:`VerifyResult` data structures
         representing the data signed by the signature, or raise an exception if the signature is not valid. By default,
         this requires the signature to be generated using a valid X.509 certificate. To enable other means of signature
-        validation, set the **require_x509** argument to `False`.
+        validation, set ``expect_config`` to a configuration with the **require_x509** parameter set to `False`.
 
         .. admonition:: See what is signed
 
@@ -294,8 +294,10 @@ class XMLVerifier(XMLSignatureProcessor):
             ``resolve_entities=False``. See https://lxml.de/FAQ.html#how-do-i-use-lxml-safely-as-a-web-service-endpoint.
         :type parser: :class:`lxml.etree.XMLParser` compatible parser
         :param uri_resolver:
-            Function to use to resolve reference URIs that don't start with "#". The function is called with a single
-            string argument containing the URI to be resolved, and is expected to return a lxml.etree node or string.
+            Function to use to resolve reference URIs that are not empty and don't start with "#" (such references are
+            only expected in detached signatures; if you don't expect such signatures, leave this unset to prevent them
+            from validating). The function is called with a single string argument containing the URI to be resolved,
+            and is expected to return a :class:`lxml.etree._Element` node or bytes.
         :param id_attribute:
             Name of the attribute whose value ``URI`` refers to. By default, SignXML will search for "Id", then "ID".
         :param expect_config:
